import osmnx as ox
import networkx as nx
import folium
import pickle
import time, datetime
import os.path
import sys
import webbrowser

 
import os


import matplotlib.pyplot as plt
plt.switch_backend('agg')


ox.settings.log_console=True
ox.settings.use_cache=True

print('OSMnx version: ', ox.__version__)
print('NetworkX version: ',nx.__version__)
print('Python version: ',sys.version)

start_time = time.time()


##
def elapsed_time(start):
    end_time = time.time()
    duration = round(end_time - start)
    timestamp = str(datetime.timedelta(seconds = duration))
    print(timestamp)


def colorize_graph(Q):
    QR = Q.copy()
    for node, dat in Q.nodes(data=True):
        d = Q.degree(node)
        dat['color'] = d

        QR.add_node(node,**dat)                #replace node value

    for u, v, k, dat in Q.edges(keys=True,data=True):
        d = 'path'
        if 'highway' in dat:
            d = dat['highway']
        dat['color']=0.0
        if isinstance(d, str):
            if d=='path' or d == 'footway':
                dat['color']=1.0
            else:
                dat['color'] = 2.0
        QR.add_edge(u,v,key=k,**dat)
    QR.graph['color']=True
    return QR

                 



     
town_markers = [(34.29963, -108.13268, 'Pie Town'),
         (33.19593, -108.20834, 'Gila Hot Springs'),
         (32.77981, -108.27533, 'Silver City'),
         (33.37939, -108.90369, 'Alma/Glenwood'),
         (33.03022, -108.16864, 'Lake Roberts'),
         (32.85594, -107.98024, 'Mimbres'),       
         (32.91685, -107.70593, 'Kingston')]

# draw() changed after moving to Linux
def draw(Q,title='',show_nodes = False, boxes=None, not_boxes = None, markers=town_markers):
    start_latlng = (33.04876, -108.30680)
    m = folium.Map(location=[start_latlng[1],start_latlng[0]],
               zoom_start=12)
    # convert to MultiDiGraph for folium
    T = Q.copy()
    if not 'color' in T.graph:
        T = colorize_graph(T)
    QD = T.to_directed()
    QD.graph['crs'] = ox.settings.default_crs

    # our graphs may be HUGE, so simplify if not already done so
    if not "simplified" in QD.graph:
        QD = ox.simplify_graph(QD, endpoint_attrs=None, remove_rings=True, track_merged=True)
    
    nodes, edges = ox.graph_to_gdfs(QD)
#    m = edges.explore(color='#ff00ff', tiles="openstreetmap",tooltip='name')
    m = edges.explore(column='color',  cmap = 'tab10',legend=False, tiles="openstreetmap",tooltip='name')
    if show_nodes:    
#        nodes.explore(m=m, color="pink", marker_kwds={"radius": 4})    
        nodes.explore(m=m, column='color', cmap='tab10', legend=False,marker_kwds={"radius": 4})    

    if not boxes == None:
        for n,s,e,w in boxes:
            folium.Rectangle(bounds=[(n,e),(n,w),(s,e),(s,w)], color='#ff7800', fill=True, fill_color='#ffff00', fill_opacity=0.2).add_to(m)
    if not not_boxes == None:
        for n,s,e,w in not_boxes:
            folium.Rectangle(bounds=[(n,e),(n,w),(s,e),(s,w)], color='#00ff78', fill=False, fill_color='#00ffff', fill_opacity=0.2).add_to(m)
    if not markers == None:
        for lat,lon, txt in markers:
            #print(lat,lon,txt)
            folium.Marker(location=[lat,lon],popup=txt,
                          icon=folium.Icon(color='lightred',icon=''),
                          tooltip=txt).add_to(m)
            
    # include the option to switch map layers
   # folium.TileLayer('openstreetmap').add_to(m)
    folium.TileLayer('cartodbpositron').add_to(m)
   # folium.TileLayer('Stamen Terrain').add_to(m)
    folium.TileLayer(
        tiles='https://{s}.tile-cyclosm.openstreetmap.fr/cyclosm/{z}/{x}/{y}.png',
        name='cyclosm',
        attr='CyclOSM',
    ).add_to(m)
    folium.LayerControl().add_to(m)    

    title_html = '''
            <h3 align="center" style="font-size:16px"><b>{}</b></h3>
             '''.format(title)   
    m.get_root().html.add_child(folium.Element(title_html))
    file_name = "/home/sagebrush/biz/graph/map_" + title + ".html"
    m.save(file_name)
    #m.show_in_browser()
    webbrowser.open(file_name)
    input("Press Enter to continue...")
    print('*************', flush=True)
    print("", flush=True)    
        
def output_gpx(Q, title='foo',markers=town_markers):
    start_latlng = (33.04876, -108.30680)
    T = Q.copy()
    QD = T.to_directed()
    QD.graph['crs'] = ox.settings.default_crs

    # our graphs may be HUGE, so simplify if not already done so
#    if not "simplified" in QD.graph:
#        QD = ox.simplify_graph(QD, endpoint_attrs=None, remove_rings=True, track_merged=True)

    ox.io.save_graph_geopackage(Q, filepath= title + '.gpkg')
    # Then we use an online service to convert between gpkg and gpx

#    nodes, edges = ox.graph_to_gdfs(QD)    
    # credit https://pygis.io/docs/d_access_osm.html
  #  edges  = edges.loc[:,edges.columns.str.contains('addr:|geometry')]
  #  edges = edges.loc[edges.geometry.type=='LineString']
  #  edges.to_file(title + '_edges.gpx', driver='GPX')

         


K = pickle.load(open('max_candidate_with_towns.pkl', 'rb'))
       
if nx.is_eulerian(K):
    print('k is eulerian!!!!!')
else:
    print('back to the drawing board')
    K = colorize_graph(K)
    draw(K,'back to drawing board',show_nodes=True)

output_gpx(K,'gxl')

            
 
elapsed_time(start_time)

print('finished script ' + os.path.basename(__file__))






